#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

"""Reimplementation of the gpiodetect tool in Python."""

import gpiod
import os

from helpers import gpio_chips

if __name__ == "__main__":
    for chip in gpio_chips():
        info = chip.get_info()
        print("{} [{}] ({} lines)".format(info.name, info.label, info.num_lines))



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

"""Reimplementation of the gpiofind tool in Python."""

import gpiod
import os
import sys

if __name__ == "__main__":
    for entry in os.scandir("/dev/"):
        if gpiod.is_gpiochip_device(entry.path):
            with gpiod.Chip(entry.path) as chip:
                offset = chip.line_offset_from_id(sys.argv[1])
                if offset is not None:
                    print("{} {}".format(chip.get_info().name, offset))
                    sys.exit(0)

    sys.exit(1)



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

"""Simplified reimplementation of the gpioset tool in Python."""

import gpiod
import sys

from gpiod.line import Direction, Value

if __name__ == "__main__":
    if len(sys.argv) < 3:
        raise TypeError(
            "usage: gpioset.py <gpiochip> <offset1>=<value1> <offset2>=<value2> ..."
        )

    path = sys.argv[1]

    def parse_value(arg):
        x, y = arg.split("=")
        return (x, Value(int(y)))

    lvs = [parse_value(arg) for arg in sys.argv[2:]]
    lines = [x[0] for x in lvs]
    values = dict(lvs)

    request = gpiod.request_lines(
        path,
        consumer="gpioset.py",
        config={tuple(lines): gpiod.LineSettings(direction=Direction.OUTPUT)},
    )

    vals = request.set_values(values)

    input()



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

import gpiod
import os


def gpio_chips():
    for path in [
        entry.path
        for entry in os.scandir("/dev/")
        if gpiod.is_gpiochip_device(entry.path)
    ]:
        with gpiod.Chip(path) as chip:
            yield chip



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

"""Simplified reimplementation of the gpiomon tool in Python."""

import gpiod
import sys

from gpiod.line import Edge

if __name__ == "__main__":
    if len(sys.argv) < 3:
        raise TypeError("usage: gpiomon.py <gpiochip> <offset1> <offset2> ...")

    path = sys.argv[1]
    lines = [int(line) if line.isdigit() else line for line in sys.argv[2:]]

    with gpiod.request_lines(
        path,
        consumer="gpiomon.py",
        config={tuple(lines): gpiod.LineSettings(edge_detection=Edge.BOTH)},
    ) as request:
        while True:
            for event in request.read_edge_events():
                print(event)



# SPDX-License-Identifier: GPL-2.0-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

from os import environ, path
from setuptools import setup, Extension, find_packages

def src(filename):
    return path.join(path.dirname(__file__), filename)

gpiod_ext = Extension(
    "gpiod._ext",
    sources=[
        src("gpiod/ext/chip.c"),
        src("gpiod/ext/common.c"),
        src("gpiod/ext/line-config.c"),
        src("gpiod/ext/line-settings.c"),
        src("gpiod/ext/module.c"),
        src("gpiod/ext/request.c"),
    ],
    define_macros=[("_GNU_SOURCE", "1")],
    libraries=["gpiod"],
    extra_compile_args=["-Wall", "-Wextra"],
)

gpiosim_ext = Extension(
    "tests.gpiosim._ext",
    sources=[src("tests/gpiosim/ext.c")],
    define_macros=[("_GNU_SOURCE", "1")],
    libraries=["gpiosim"],
    extra_compile_args=["-Wall", "-Wextra"],
)

procname_ext = Extension(
    "tests.procname._ext",
    sources=[src("tests/procname/ext.c")],
    define_macros=[("_GNU_SOURCE", "1")],
    extra_compile_args=["-Wall", "-Wextra"],
)

extensions = [gpiod_ext]
if "GPIOD_WITH_TESTS" in environ and environ["GPIOD_WITH_TESTS"] == "1":
    extensions.append(gpiosim_ext)
    extensions.append(procname_ext)

with open(src("gpiod/version.py"), "r") as fd:
    exec(fd.read())

setup(
    name="gpiod",
    packages=find_packages(include=["gpiod"]),
    ext_modules=extensions,
    version=__version__,
    author="Bartosz Golaszewski",
    author_email="brgl@bgdev.pl",
    description="Python bindings for libgpiod",
    platforms=["linux"],
    license="LGPLv2.1",
)



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-or-later
# SPDX-FileCopyrightText: 2022 Bartosz Golaszewski <brgl@bgdev.pl>

"""Simplified reimplementation of the gpioinfo tool in Python."""

import gpiod
import os

from helpers import gpio_chips

if __name__ == "__main__":
    for chip in gpio_chips():
        cinfo = chip.get_info()
        print("{} - {} lines:".format(cinfo.name, cinfo.num_lines))

        for offset in range(0, cinfo.num_lines):
            linfo = chip.get_line_info(offset)
            is_input = linfo.direction == gpiod.line.Direction.INPUT
            print(
                "\tline {:>3}: {:>18} {:>12} {:>8} {:>10}".format(
                    linfo.offset,
                    linfo.name or "unnamed",
                    linfo.consumer or "unused",
                    "input" if is_input else "output",
                    "active-low" if linfo.active_low else "active-high",
                )
            )
